import React, { forwardRef, useImperativeHandle, useRef, useState } from 'react';
import { StyleSheet } from 'react-native';
import { WebView } from 'react-native-webview';
import * as ExposeMethods from './exposeMethods';
import * as Scripts from './scripts';
import { ExposeOTPVerificationRefProps, ExposeMethodResponse } from './types'

const url = "https://control.msg91.com/app/assets/otp-provider/otp-provider.js";

interface Props {
  authToken: string,
  widgetId: string,
  getWidgetData?: (data: any) => void
}

const ExposeOTPVerification: React.ForwardRefRenderFunction<ExposeOTPVerificationRefProps, Props> = ({
  authToken,
  widgetId,
  getWidgetData
}, ref) => {
  const webviewRef = useRef<WebView>();
  const [retryCount, setRetryCount] = useState(0);
  const [requestPromises, setRequestPromises] = useState<any>({});

  const sendOtp = (input : string) => {
    let promiseResolve: any, promiseReject: any;
    
    const promise = new Promise<ExposeMethodResponse>((resolve, reject) => {
      promiseResolve = resolve;
      promiseReject = reject;
      setRequestPromises({...requestPromises, 'sendOtp' : {resolve, reject}})
    });

    webviewRef?.current?.injectJavaScript(ExposeMethods.sendOtp(input));

    // Automatically resolves promise with custom error message if did not receive response within 5 seconds
    const timeoutTimer = setTimeout(() => {
      const data = {
        message: 'Retry after sometime',
        type: 'custom_timeout_error'
      }
      promiseResolve(data);
      setRequestPromises((promises) => {
        delete promises['sendOtp'];
        return promises;
      });
    }, 10000);

    promise.finally(() => {
      clearTimeout(timeoutTimer);
    });

    return promise;
  }

  const retryOtp = (retryOn? : 'SMS' | 'VOICE' | 'EMAIL' | 'WHATSAPP') => {
    let promiseResolve, promiseReject;

    const promise = new Promise<ExposeMethodResponse>((resolve, reject) => {
      promiseResolve = resolve;
      promiseReject = reject;
      setRequestPromises({...requestPromises, 'retryOtp' : {resolve, reject}})
    });
    
    webviewRef?.current?.injectJavaScript(ExposeMethods.retryOtp(retryOn));

    // Automatically resolves promise with custom error message if did not receive response within 5 seconds
    const timeoutTimer = setTimeout(() => {
      const data = {
        message: 'Retry after sometime',
        type: 'custom_timeout_error'
      }
      promiseResolve(data);
      setRequestPromises((promises) => {
        delete promises['retryOtp'];
        return promises;
      });
    }, 10000);

    promise.finally(() => {
      clearTimeout(timeoutTimer);
    });

    return promise;
  }

  const verifyOtp = (otp : number) => {
    let promiseResolve, promiseReject;

    const promise = new Promise<ExposeMethodResponse>((resolve, reject) => {
      promiseResolve = resolve;
      promiseReject = reject;
      setRequestPromises({...requestPromises, 'verifyOtp' : {resolve, reject}})
    });

    webviewRef?.current?.injectJavaScript(ExposeMethods.verifyOtp(otp));

    // Automatically resolves promise with custom error message if did not receive response within 5 seconds
    const timeoutTimer = setTimeout(() => {
      const data = {
        message: 'Retry after sometime',
        type: 'custom_timeout_error'
      }
      promiseResolve(data);
      setRequestPromises((promises) => {
        delete promises['verifyOtp'];
        return promises;
      });
    }, 10000);

    promise.finally(() => {
      clearTimeout(timeoutTimer);
    });

    return promise;
  }

  useImperativeHandle(
    ref, 
    () => ({
      sendOtp: sendOtp,
      retryOtp: retryOtp,
      verifyOtp: verifyOtp
    }), 
    [sendOtp, retryOtp, verifyOtp]
  );

  const onMessage = (data) => {
    const response = JSON.parse(data.nativeEvent.data)

    if(response?.type == 'error' && response?.code == '201' && response?.message == 'AuthenticationFaliure'){
      if(retryCount == 0){
        webviewRef.current?.reload();
        setRetryCount(1);
        return;
      }
      console.error('OTP Widget: ', response);
    }

    if(response?.widgetId && response?.companyId){
      if(typeof getWidgetData === "function"){
        getWidgetData(response);
      }
      return;
    }

    if(requestPromises['sendOtp']){

      const { resolve, reject } = requestPromises['sendOtp'];
      resolve(response);
      const promises = requestPromises;
      delete promises['sendOtp']
      setRequestPromises(promises);

    } else if(requestPromises['verifyOtp']){

      const { resolve, reject } = requestPromises['verifyOtp'];
      resolve(response);
      const promises = requestPromises;
      delete promises['verifyOtp']
      setRequestPromises(promises);

    } else if(requestPromises['retryOtp']){

      const { resolve, reject } = requestPromises['retryOtp'];
      resolve(response);
      const promises = requestPromises;
      delete promises['retryOtp']
      setRequestPromises(promises);

    }
  }

  return (
    <WebView
      androidLayerType="software"
      webviewDebuggingEnabled
      ref={webviewRef}
      containerStyle={styles.containerStyle}
      style={styles.style}
      onMessage={onMessage}
      originWhitelist={["*"]}
      javaScriptEnabled={true}
      source={{ 
        html: Scripts.exposeVerificationScript(widgetId, authToken),
        baseUrl: url
      }} 
    />
  )
   
}

const styles = StyleSheet.create({
  containerStyle: {
    position: 'absolute', 
    height: 1, 
    width: 1
  },
  style: {
    opacity: 0.1
  }
})


/**
 * MSG91 Expose OTP Widget component
 * 
 * It is used to integrate the OTP Widget within your own custom UI
 * 
 * @see https://www.npmjs.com/package/@msg91comm/react-native-sendotp
 */
export default forwardRef(ExposeOTPVerification);